// RTM API Key
var myKey = "552a66e3588ffbe4e7811282ba9a09ef";

// Frob needs to be shared by auth functions
var frob = null;

// auth token
var myToken = null; 

/* this is just a reference to the content
of the scrollarea */
var taskArea;

// array of task objects
var taskList; 

// RTM timeline
var timeline = null;

// The last time we updated
var lastSync = null;

// the last transaction (for undo)
var lastTransaction = null;

// all lists (for the list picker on the back)
var lists = null;

// only get tasks from this list (default is 0, all lists)
var onlyFromList = 0;

function load()
{
	setupParts();
	
	// load prefs
	myToken = widget.preferenceForKey("token");
	taskArea = document.getElementById("content");
	taskList = new Array();
	
	if ( widget.preferenceForKey("onlyFromList") > 0 ) {
		onlyFromList = widget.preferenceForKey("onlyFromList");
	}
	
	/* sucks to do this here but the text doesn't align correctly if it's set to hidden in the 
	Dashcode UI */
	document.getElementById("undo").style.display = "none";
}

function remove()
{
	// remove our preferences
	widget.setPreferenceForKey( null, "token" );
	widget.setPreferenceForKey( null, "onlyFromList" );
}

function hide()
{
	// remove last transaction (showing undo button across hides is confusing)
	lastTransaction = null;
	document.getElementById("undo").style.display = "none";
}

function show()
{
	// clear error display
	document.getElementById("netError").style.display = "none";
	
	// see if we have an auth token
	 if( myToken == null) {
		auth();
		return;
	 }
	
	// see if we have a timeline
	if( timeline == null ) {
		getTimeLine();
		return;
	}
	
	// we need the lists for the picker on the back
	if( lists == null ) {
		getLists();
		return;
	}

	// allright then, get the task list
	getTasks();
}

function showBack(event)
{
    // Preselect correct item in dropdown list
	if ( onlyFromList != 0 ) {
		select = document.getElementById("selectList");
		for ( var i = 0; i < selectList.length; i++ ) {
			if ( select.options[i].value == onlyFromList ) {
				select.options[i].selected = true;
				break;
			}
		}
	}
	
	// show the back

	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToBack");

	front.style.display="none";
	back.style.display="block";
	
	if (window.widget)
		setTimeout('widget.performTransition();', 0);
}

function showFront(event)
{
	// if a change was made to which list to show tasks from
	select = document.getElementById("selectList");
	if ( select.value != onlyFromList ) {
		onlyFromList = select.value;
		widget.setPreferenceForKey( onlyFromList, "onlyFromList" );
		
		// need to throw out last sync and re-sync if list changes
		lastSync = null;
		taskList = new Array();
		show();
	}

	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToFront");

	front.style.display="block";
	back.style.display="none";
	
	if (window.widget)
		setTimeout('widget.performTransition();', 0);
}

if (window.widget)
{
	widget.onremove = remove;
	widget.onhide = hide;
	widget.onshow = show;
}

// this is called if there's no valid token
function auth() {	
	if (frob == null) { // no frob so show auth link and bail
		document.getElementById("noAuth").style.display = "block"; // inform user
		document.getElementById("clickToAuth").style.display = "block";
		return;
	}
		
	// we have a frob, try to get a token with it
	spinner("start");
	requestToken(); 
}

// this function responds to a click on the "authenticate" link
function authClicked (event) {
	// hid the click to auth text, show spinner instead
	document.getElementById("noAuth").style.display = "none";
	document.getElementById("clickToAuth").style.display = "none";
	spinner("start");
    
	// this array will hold our parameters for the frob request
	var reqParams = new Object();

	reqParams.method = "rtm.auth.getFrob";
	reqParams.api_key = myKey;

	// sign it
	signParams(reqParams);

	// make the http request
	request('http://www.rememberthemilk.com/services/rest/?', reqParams, openAuthURL);
}

// Callback function for rtm.auth.getFrob which opens an auth URL using the frob
function openAuthURL (http_request) {
	// only if req shows "loaded"
    if (http_request.readyState != 4) { return; }
	
	// check for error
	if (http_request.status != 200 ) {
		spinner("stop");
		document.getElementById('netError').style.display = "block";
		return;
	}
	
	// if we got here all is well: extract the frob
	frob = http_request.responseXML.getElementsByTagName("frob")[0].childNodes[0].nodeValue;

	// build and open a signed auth URL
	reqParams = new Object();
	reqParams.api_key = myKey;
	reqParams.frob = frob;
	reqParams.perms = "delete";
	
	// sign it
	signParams(reqParams);
	
	// build the URL
	var url = 'http://www.rememberthemilk.com/services/auth/?';
	for (var paramName in reqParams) {
		url += (paramName + "=" + reqParams[paramName] + "&");
	}
	
	// kill spinner
	spinner("stop");
	widget.openURL(url);
}

// get an authentication token from RTM
function requestToken () {

    // request parameters, including the frob we got earlier
	reqParams = new Object();
	reqParams.method = "rtm.auth.getToken";
	reqParams.api_key = myKey;
	reqParams.frob = frob;
	signParams(reqParams);

	// make the request
	request('http://www.rememberthemilk.com/services/rest/?', reqParams, saveToken);
}

// callback function to extract and save the auth token
function saveToken( http_request ) {
	// only if req shows "loaded"
    if (http_request.readyState != 4) { return; }
	
	// check for error
	if (http_request.status != 200 ) {
		spinner("stop");
		document.getElementById('netError').style.display = "block";
		return;
	}
	
	// if we got here it succeeded; save the token
	myToken = http_request.responseXML.getElementsByTagName("token")[0].childNodes[0].nodeValue;
	widget.setPreferenceForKey(myToken, "token");
	
	spinner("stop");
	
	// refresh the interface
	show();
}

// get a timeline from RTM
function getTimeLine() { 
	spinner("start");
	// request parameters
	reqParams = new Object();
	reqParams.method = "rtm.timelines.create";
	reqParams.api_key = myKey;
	reqParams.auth_token = myToken;
	signParams(reqParams);
	
	request('http://www.rememberthemilk.com/services/rest/?', reqParams, saveTimeLine);
}

// callback function to save the timeline we get from RTM
function saveTimeLine ( http_request ) {
	// only if req shows "loaded"
    if (http_request.readyState != 4) { return; }
	
	// check for error
	if (http_request.status != 200 ) {
		spinner("stop");
		document.getElementById('netError').style.display = "block";
		return;
	}
	
	// if we got here all is well: extract and save the timeline value
	timeline = http_request.responseXML.getElementsByTagName("timeline")[0].childNodes[0].nodeValue;
	spinner("stop");
	show();
}

// function to get all the user's lists: we need these in case they want to display tasks from only one list
function getLists() {
spinner("start");
	// request parameters
	reqParams = new Object();
	reqParams.method = "rtm.lists.getList";
	reqParams.api_key = myKey;
	reqParams.auth_token = myToken;
	signParams(reqParams);
	
	request('http://www.rememberthemilk.com/services/rest/?', reqParams, saveLists);
}

// callback function fpr rtm.lists.getList
function saveLists ( http_request ) {
	// only if req shows "loaded"
    if (http_request.readyState != 4) { return; }
	
	// check for error
	if (http_request.status != 200 ) {
		spinner("stop");
		document.getElementById('netError').style.display = "block";
		return;
	}
	
	// if we got here all is well
	var selectList = document.getElementById("selectList");
	lists = http_request.responseXML.getElementsByTagName("list");
	for ( var i = 0; i < lists.length; i++ ) {
		thisList = lists[i];
		
		if ( thisList.getAttribute("deleted") == 1 ) { continue; }
		
		var listOpt = document.createElement('option');
		listOpt.text = thisList.getAttribute("name");
		listOpt.value = thisList.getAttribute("id");
		selectList.add( listOpt );
	}
	
	spinner("stop");
	show();
}

// request task list from RTM, optionally limited to those modified since last_sync
function getTasks() {
	spinner("start");
	
	// request parameters
	reqParams = new Object();
	reqParams.method = "rtm.tasks.getList";
	reqParams.api_key = myKey;
	reqParams.auth_token = myToken;
	
	// if we already got some tasks, only get the updated ones
	if ( lastSync != null ) { reqParams.last_sync = lastSync }
	
	// if the user set the preference to only display tasks from a certain list
	if ( onlyFromList != 0 ) { reqParams.list_id = onlyFromList }
		
	signParams(reqParams);
	
	request('http://www.rememberthemilk.com/services/rest/?', reqParams, showTasks);
}

// callback function to extract and display tasks
function showTasks( http_request ) {
	// only if req shows "loaded"
    if (http_request.readyState != 4) { return; }
	
	// check for error
	if (http_request.status != 200 ) {
		spinner("stop");
		document.getElementById('tasks').style.display = "none";
		document.getElementById('netError').style.display = "block";
		return;
	}
	
	// save the last sync time
	var now = new Date();
	lastSync = now.toISO8601String("5");
	
	taskLists = http_request.responseXML.getElementsByTagName('list');
	for ( var list = 0; list < taskLists.length; list++ ) {
		thisList = taskLists[list];
		for ( var series = 0; series < thisList.childNodes.length; series++ ) {
			var thisSeries = thisList.childNodes(series);
			var tasks = thisSeries.getElementsByTagName("task");
				
			for ( var i = 0; i < tasks.length; i++ ) {
				thisTask = tasks[i];
				
				// see if we're syncing existing tasks
				if ( thisList.getAttribute("current") != null && getTaskPos( thisTask.id ) >= 0 ) {
					removeTask( thisTask.id );
				}
						
				if ( thisTask.getAttribute("completed") == "" ) {
					task = document.createElement("div");
					task.setAttribute( 'taskName', thisSeries.getAttribute('name') );
					task.setAttribute( 'id' , thisTask.getAttribute('id') );
					task.setAttribute( 'seriesID' , thisSeries.getAttribute('id') );
					task.setAttribute( 'listID', thisList.getAttribute('id') );
					task.setAttribute( 'due', thisTask.getAttribute('due') );
					task.setAttribute( 'class', 'task' );
					
					var checkBox = document.createElement('div');
					checkBox.setAttribute('class', 'checkBox');
					checkBox.setAttribute('onClick', 'taskClick( this )');
					
					task.appendChild(checkBox);	
				
					var dueSpan = document.createElement('span');
					dueSpan.setAttribute('class', 'due');
					
					if ( task.getAttribute("due") != "" ) { 
						dueSpan.innerText = dueDateToString( task.getAttribute("due") )
						task.appendChild(dueSpan)
					}
					
					var nameSpan = document.createElement('span');
					nameSpan.setAttribute('class', 'name');
					nameSpan.innerText = task.getAttribute("taskName");
					task.appendChild(nameSpan);
					
					taskList.push( task );
					
				}
			}
		}
	}
	
	taskList.sort(sortByDueDate);
	 
	spinner("stop");
	
	if ( taskList.length == 0 ) {
		// no tasks to show
		document.getElementById("notasks").style.display = "block";
	} else { 
		document.getElementById("notasks").style.display = "none";
		
		// clear out the taskArea and repopulate
		while ( taskArea.hasChildNodes() ) { taskArea.removeChild( taskArea.firstChild ); }
		for ( var i = 0; i < taskList.length; i++ ) {
			// "true" parameter copies children too
			task = taskList[i].cloneNode(true);
			taskArea.appendChild( task );
		}
		document.getElementById("tasks").style.display = "block";
		document.getElementById("tasks").object.refresh();	
	}
}
	
// this function is called when a checkbox is clicked to complete a task	
function taskClick( checkBox ) {
	checkBox.style.backgroundImage = 'url(Images/check.png)';
	task = checkBox.parentNode;
		
	spinner("start");
	// request parameters
	reqParams = new Object();
	reqParams.method = "rtm.tasks.complete";
	reqParams.api_key = myKey;
	reqParams.auth_token = myToken;
	reqParams.timeline = timeline;
	reqParams.list_id = task.getAttribute("listID");
	reqParams.taskseries_id = task.getAttribute("seriesID");
	reqParams.task_id = task.getAttribute("id");
	signParams(reqParams);
	
	request('http://www.rememberthemilk.com/services/rest/?', reqParams, markTaskComplete);
}

// callback function for request to rtm.tasks.complete
function markTaskComplete( http_request ) { 
	// only if req shows "loaded"
    if (http_request.readyState != 4) { return; }
	
	// check for error
	if (http_request.status != 200 ) {
		spinner("stop");
		document.getElementById('tasks').style.display = "none";
		document.getElementById('netError').style.display = "block";
		return;
	}

	// if we got here we're all good
	spinner("stop");
	
	// save the transaction ID & show undo button
	var transaction = http_request.responseXML.getElementsByTagName('transaction');
	lastTransaction = transaction[0].getAttribute("id");
	document.getElementById("undo").style.display = "block";
	
	// get tasks
	var tasks = http_request.responseXML.getElementsByTagName('task');
	taskID = tasks[0].getAttribute("id");
	
	// remove the task from the task list
	removeTask( taskID );
	
	/* minor optimization: remove the task from the taskArea ourselves so we
	done't have to do a full refresh */
	task = document.getElementById( taskID );
	taskArea.removeChild( task );
	document.getElementById("tasks").object.refresh();	
	
}

// function to undo last transaction, called when undo button is clicked
function undoLast( event ) {
	spinner("start");
	// request parameters
	reqParams = new Object();
	reqParams.method = "rtm.transactions.undo";
	reqParams.api_key = myKey;
	reqParams.auth_token = myToken;
	reqParams.timeline = timeline;
	reqParams.transaction_id = lastTransaction;
	signParams(reqParams);
	
	request('http://www.rememberthemilk.com/services/rest/?', reqParams, undoCallback);
}

// callback function for rtm.transactions.undo
function undoCallback() {
	// only if req shows "loaded"
    if (http_request.readyState != 4) { return; }
	
	// check for error
	if (http_request.status != 200 ) {
		spinner("stop");
		document.getElementById('tasks').style.display = "none";
		document.getElementById("undo").style.display = "none";
		document.getElementById('netError').style.display = "block";
		return;
	}

	// if we got here we're all good
	spinner("stop");
	document.getElementById("undo").style.display = "none";
	show();
}

// Utility functions to manage the tasklist
function getTaskPos( task_id ) {
	for ( var i = 0; i < taskList.length; i++ ) {
		if ( taskList[i].getAttribute("id") == task_id ) { return i; }
	}
	return -1;
}

function removeTask( task_id ) {
	for ( var i = 0; i < taskList.length; i++ ) {
		if ( taskList[i].getAttribute("id") == task_id ) {
			taskList.splice(i,1);
			return; 
		}
	}
}

/* utility funtion to handle XMLHttpRequest. It takes a url string, an associative array of parameters,
and a callback function name as arguments */
function request(url, params, callback_func ) {

   http_request = false;
   http_request = new XMLHttpRequest();
   if (http_request.overrideMimeType) {
   	http_request.overrideMimeType('text/xml');
   }
   
   if (!http_request) {
    return false;
   }
   
   //build the url
   for (var paramName in params) {
		url += (paramName + "=" + params[paramName] + "&");
	}

   
   http_request.onreadystatechange = function() { callback_func(http_request); };
   http_request.open('GET', url, true); 
   
   http_request.send(null);
}


/* Sign an associative array of request parameters. To compute the signatures we sort alphabetically
by key name, concatenate keys and values, and then append that to our secret key. We then compute an
MD5 hash of that string and insert it into the associative array under the key name api_sig */
function signParams(parameters) {
	// secret key from RTM
	var paramString = "8a1d99aad9d1edf6";
	
	// create an index array that holds our key values and then sort it
	var paramsIndex = new Array();
	for (var param in parameters) {
		paramsIndex.push(param);
	}
	paramsIndex.sort();
	
	// this pulls out the request parameters in alpha order and concatenates them
	for (var param in paramsIndex) {
		var paramName = paramsIndex[param];
		var paramValue = parameters[paramName];
		paramString += (paramName + paramValue);
	}
	
	// add the api_sig parameter to the array
	parameters.api_sig = MD5(paramString);
}

// start and stop the busy spinner
function spinner ( command ) {
	switch ( command ) {
		case "start":
			document.getElementById("spinner").style.display = "block"; 
			break
		case "stop":
			document.getElementById("spinner").style.display = "none"; 
			break
	}
}

/* utility function to compare due dates of two task objects:
due sooner is "smaller," so tasks that are due sooner sort to the 
beginning of the list. tasks with no due date sort to the end */
function sortByDueDate ( task1, task2 ) {
	var task1_due = task1.getAttribute("due");
	var task2_due = task2.getAttribute("due");
	
	if ( task1_due == "" && task2_due == "" ) { return 0; }
	if ( task1_due == "" ) { return 1; }
	if ( task2_due == "" ) { return -1; }
	
	var due1 = new Date();
	var due2 = new Date();
	
	due1.setISO8601( task1_due );
	due2.setISO8601( task2_due );
	
	if ( due1 > due2 ) { return 1; }
	if (due1 < due2 ) { return -1; }
	
	return 0;
}
 
/* parse an ISO 8601 date string and return "Overdue," "Due today," or "Due Mon Day" 
depending on what the due date is */
function dueDateToString ( dueDate ) {
	var now = new Date();
	todayStart = now.getTime() - (now.getHours()*1000*60*60) - (now.getMinutes()*1000*60) - (now.getSeconds()*1000) - now.getMilliseconds();
	
	var due = new Date();
	due.setISO8601( dueDate );
	
	if ( due.getTime() < todayStart ) { return "Overdue"; }
	if ( due.getTime() < (todayStart + 86400000) ) { return "Due today"; }
	
	var month;
	switch ( due.getMonth() ) {
		case 0: month = "Jan"; break;
		case 1: month = "Feb"; break;
		case 2: month = "Mar"; break;
		case 3: month = "Apr"; break;
		case 4: month = "May"; break;
		case 5: month = "Jun"; break;
		case 6: month = "Jul"; break;
		case 7: month = "Aug"; break;
		case 8: month = "Sep"; break;
		case 9: month = "Oct"; break;
		case 10: month = "Nov"; break;
		case 11: month = "Dec"; break;
	}
	
	return( "Due " + month + " " + due.getDate() );
}